@@ -251,12 +251,14 @@ Status CheckInputs(const T* query,
                            "seqlens_k must be shape (batch_size).");
   }
 
-  // Set present sequence length from input total_seqlen tensor
   if (!onnxruntime::IsScalarOr1ElementVector(total_seqlen)) {
     return ORT_MAKE_STATUS(ONNXRUNTIME, INVALID_ARGUMENT,
                            "total_sequence_length tensor must be of one element.");
   }
-  int total_sequence_length = *((*total_seqlen).template Data<int32_t>());
+
+  // When graph capture is enabled, total_seqlen is on GPU and cannot be read. Skip validation.
+  const bool is_total_seqlen_on_cpu = (total_seqlen->Location().device.Type() == OrtDevice::CPU);
+  int total_sequence_length = is_total_seqlen_on_cpu ? *((*total_seqlen).template Data<int32_t>()) : 0;
   int present_sequence_length = std::max(total_sequence_length, past_sequence_length);
 
   int rotary_dim = 0;
@@ -267,22 +269,20 @@ Status CheckInputs(const T* query,
                            "Input 'cos_cache' and 'sin_cache' shall be both present or both absent.");
   }
 
+  // Skip prompt type detection when total_seqlen is on GPU (graph capture mode)
   bool is_subsequent_prompt = false;
-  if (sequence_length > 1 && sequence_length != total_sequence_length) {
-    if (batch_size != 1) {
-      return ORT_MAKE_STATUS(ONNXRUNTIME, INVALID_ARGUMENT,
-                             "batch_size must be 1 when sequence_length > 1 and past context is given.");
+  bool is_first_prompt = false;
+  if (is_total_seqlen_on_cpu) {
+    if (sequence_length > 1 && sequence_length != total_sequence_length) {
+      if (batch_size != 1) {
+        return ORT_MAKE_STATUS(ONNXRUNTIME, INVALID_ARGUMENT,
+                               "batch_size must be 1 when sequence_length > 1 and past context is given.");
+      }
+      is_subsequent_prompt = true;
     }
-    is_subsequent_prompt = true;
-  }
 
-  bool is_first_prompt;
-  if (is_subsequent_prompt) {
-    is_first_prompt = false;  // irrelevant for interactive decoding
-  } else {
-    // If not interactive, sequence_length is 1 for token gen and arbitrarily large for prompt
     is_first_prompt = (sequence_length == total_sequence_length);
-    if (!is_first_prompt && sequence_length != 1) {
+    if (!is_subsequent_prompt && !is_first_prompt && sequence_length != 1) {
       return ORT_MAKE_STATUS(ONNXRUNTIME, INVALID_ARGUMENT,
                              "sequence_length shall be 1 when it is not prompt.");
     }
@@ -351,7 +351,7 @@ Status CheckCustomAttentionInputs(const T* position_ids,
 
     if (pos_ids_shape[1] < parameters.sequence_length) {
       return ORT_MAKE_STATUS(ONNXRUNTIME, INVALID_ARGUMENT,
-                             "position_ids dimension 1 must be atleast sequence length, got ", pos_ids_shape[1]);
+                             "position_ids dimension 1 must be at least sequence length, got ", pos_ids_shape[1]);
     }
   }
 
