@@ -125,10 +125,29 @@ ONNX_CPU_OPERATOR_VERSIONED_KERNEL(
     If);
 
 // Opset 23 added support for float4e2m1.
-// TODO(titaiwang): Add support for float4e2m1.
-ONNX_CPU_OPERATOR_KERNEL(
+// TODO: Add support for float4e2m1.
+ONNX_CPU_OPERATOR_VERSIONED_KERNEL(
     If,
     23,
+    23,
+    KernelDefBuilder()
+        .TypeConstraint("B", DataTypeImpl::GetTensorType<bool>())
+        .TypeConstraint("V", DataTypeImpl::AllTensorAndSequenceTensorAndOptionalTypesIRv9()),
+    If);
+
+ONNX_CPU_OPERATOR_VERSIONED_KERNEL(
+    If,
+    24,
+    24,
+    KernelDefBuilder()
+        .TypeConstraint("B", DataTypeImpl::GetTensorType<bool>())
+        .TypeConstraint("V", DataTypeImpl::AllTensorAndSequenceTensorAndOptionalTypesIRv9()),
+    If);
+
+// Opset 25
+ONNX_CPU_OPERATOR_KERNEL(
+    If,
+    25,
     KernelDefBuilder()
         .TypeConstraint("B", DataTypeImpl::GetTensorType<bool>())
         .TypeConstraint("V", DataTypeImpl::AllTensorAndSequenceTensorAndOptionalTypesIRv9()),
